@@ -1,7 +1,35 @@
-use crate::SyntaxKind;
 use std::iter::Peekable;
 use std::str::Chars;
 
+/// Let's start with defining all kinds of tokens and
+/// composite nodes.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[allow(non_camel_case_types)]
+#[repr(u16)]
+pub enum SyntaxKind {
+    KEY = 0,
+    VALUE,
+    COLON,
+    INDENT,
+    NEWLINE,
+    WHITESPACE, // whitespaces is explicit
+    COMMENT,    // comments
+    ERROR,      // as well as errors
+
+    // composite nodes
+    ROOT,       // The entire file
+    PARAGRAPH,  // A deb822 paragraph
+    ENTRY,      // A single key-value pair
+    EMPTY_LINE, // An empty line
+}
+
+/// Convert our `SyntaxKind` into the rowan `SyntaxKind`.
+impl From<SyntaxKind> for rowan::SyntaxKind {
+    fn from(kind: SyntaxKind) -> Self {
+        Self(kind as u16)
+    }
+}
+
 pub struct Lexer<'a> {
     input: Peekable<Chars<'a>>,
     start_of_line: bool,
@@ -19,7 +47,18 @@ impl<'a> Lexer<'a> {
         }
     }
 
+    pub fn new_inline(input: &'a str) -> Self {
+        Lexer {
+            input: input.chars().peekable(),
+            start_of_line: false,
+            colon_count: 1,
+            indent: 0,
+        }
+    }
+
     fn is_whitespace(c: char) -> bool {
+        // deb822(5) says that continuation lines
+        // start with a space (U+0020) or a tab (U+0009).
         c == ' ' || c == '\t'
     }
 
@@ -28,7 +67,11 @@ impl<'a> Lexer<'a> {
     }
 
     fn is_valid_key_char(c: char) -> bool {
-        c.is_ascii_alphanumeric() || c == '-' || c == '_' || c == '.'
+        // deb822(5) says valid field characters are US-ASCII
+        // characters excluding control characters, space and colon
+        // (i.e. characters in the ranges U+0021 to U+0039 and U+003B to U+007E).
+        // I.e. printable characters except for space and colon.
+        c.is_ascii_graphic() && c != ':' && c != ' '
     }
 
     fn read_while<F>(&mut self, predicate: F) -> String
@@ -78,7 +121,11 @@ impl<'a> Lexer<'a> {
                     self.colon_count = 0;
                     Some((SyntaxKind::COMMENT, format!("#{}", comment)))
                 }
-                _ if Self::is_valid_key_char(c) && self.start_of_line && self.indent == 0 => {
+                _ if Self::is_valid_key_char(c)
+                    && c != '-'
+                    && self.start_of_line
+                    && self.indent == 0 =>
+                {
                     let key = self.read_while(Self::is_valid_key_char);
                     self.start_of_line = false;
                     Some((SyntaxKind::KEY, key))
@@ -99,7 +146,7 @@ impl<'a> Lexer<'a> {
 }
 
 impl Iterator for Lexer<'_> {
-    type Item = (crate::SyntaxKind, String);
+    type Item = (SyntaxKind, String);
 
     fn next(&mut self) -> Option<Self::Item> {
         self.next_token()
@@ -111,9 +158,14 @@ pub(crate) fn lex(input: &str) -> Vec<(SyntaxKind, String)> {
     lexer.by_ref().collect::<Vec<_>>()
 }
 
+pub(crate) fn lex_inline(input: &str) -> Vec<(SyntaxKind, String)> {
+    let mut lexer = Lexer::new_inline(input);
+    lexer.by_ref().collect::<Vec<_>>()
+}
+
 #[cfg(test)]
 mod tests {
-    use crate::SyntaxKind::*;
+    use super::SyntaxKind::*;
     #[test]
     fn test_empty() {
         assert_eq!(super::lex(""), vec![]);
@@ -227,6 +279,7 @@ Files:
  b7a7d67a02974c52c408fdb5e118406d 890 cvsd_1.0.24.dsc
  b73ee40774c3086cb8490cdbb96ac883 258139 cvsd_1.0.24.tar.gz
 Vcs-Browser: http://arthurdejong.org/viewvc/cvsd/
+# A comment
 Vcs-Cvs: :pserver:anonymous@arthurdejong.org:/arthur/
 Checksums-Sha256:
  a7bb7a3aacee19cd14ce5c26cb86e348b1608e6f1f6e97c6ea7c58efa440ac43 890 cvsd_1.0.24.dsc
@@ -254,7 +307,9 @@ Section: vcs
                 (KEY, "Architecture"), (COLON, ":"), (WHITESPACE, " "), (VALUE, "any"), (NEWLINE, "\n"),
                 (KEY, "Standards-Version"), (COLON, ":"), (WHITESPACE, " "), (VALUE, "3.9.3"), (NEWLINE, "\n"),
                 (KEY, "Format"), (COLON, ":"), (WHITESPACE, " "), (VALUE, "3.0 (native)"), (NEWLINE, "\n"),
-                (KEY, "Files"), (COLON, ":"), (NEWLINE, "\n"), (INDENT, " "), (VALUE, "b7a7d67a02974c52c408fdb5e118406d 890 cvsd_1.0.24.dsc"), (NEWLINE, "\n"), (INDENT, " "), (VALUE, "b73ee40774c3086cb8490cdbb96ac883 258139 cvsd_1.0.24.tar.gz"), (NEWLINE, "\n"), (KEY, "Vcs-Browser"), (COLON, ":"), (WHITESPACE, " "), (VALUE, "http://arthurdejong.org/viewvc/cvsd/"), (NEWLINE, "\n"),
+                (KEY, "Files"), (COLON, ":"), (NEWLINE, "\n"), (INDENT, " "), (VALUE, "b7a7d67a02974c52c408fdb5e118406d 890 cvsd_1.0.24.dsc"), (NEWLINE, "\n"), (INDENT, " "), (VALUE, "b73ee40774c3086cb8490cdbb96ac883 258139 cvsd_1.0.24.tar.gz"), (NEWLINE, "\n"),
+                (KEY, "Vcs-Browser"), (COLON, ":"), (WHITESPACE, " "), (VALUE, "http://arthurdejong.org/viewvc/cvsd/"), (NEWLINE, "\n"),
+                (COMMENT, "# A comment"), (NEWLINE, "\n"),
                 (KEY, "Vcs-Cvs"), (COLON, ":"), (WHITESPACE, " "), (VALUE, ":pserver:anonymous@arthurdejong.org:/arthur/"), (NEWLINE, "\n"),
                 (KEY, "Checksums-Sha256"), (COLON, ":"), (NEWLINE, "\n"), (INDENT, " "), (VALUE, "a7bb7a3aacee19cd14ce5c26cb86e348b1608e6f1f6e97c6ea7c58efa440ac43 890 cvsd_1.0.24.dsc"), (NEWLINE, "\n"), (INDENT, " "), (VALUE, "46bc517760c1070ae408693b89603986b53e6f068ae6bdc744e2e830e46b8cba 258139 cvsd_1.0.24.tar.gz"), (NEWLINE, "\n"),
                 (KEY, "Homepage"), (COLON, ":"), (WHITESPACE, " "), (VALUE, "http://arthurdejong.org/cvsd/"), (NEWLINE, "\n"),
@@ -265,4 +320,38 @@ Section: vcs
             ]
         );
     }
+
+    #[test]
+    fn test_lex_inline() {
+        let text = r"syncthing-gtk";
+        let tokens = super::lex_inline(text);
+        assert_eq!(
+            tokens
+                .iter()
+                .map(|(kind, text)| (*kind, text.as_str()))
+                .collect::<Vec<_>>(),
+            vec![(VALUE, "syncthing-gtk")]
+        );
+    }
+
+    #[test]
+    fn test_lex_odd_key_characters() {
+        let text = "foo-bar: baz\n";
+
+        let tokens = super::lex(text);
+
+        assert_eq!(
+            tokens
+                .iter()
+                .map(|(kind, text)| (*kind, text.as_str()))
+                .collect::<Vec<_>>(),
+            vec![
+                (KEY, "foo-bar"),
+                (COLON, ":"),
+                (WHITESPACE, " "),
+                (VALUE, "baz"),
+                (NEWLINE, "\n")
+            ]
+        );
+    }
 }
